@@ -79,13 +79,18 @@ def send_rpc(connection, context, exchange, topic, method, args,
 
 
 def reply(connection, msg_id, replydata=None, failure=None, on_return=None):
-    if on_return is not None:
-        raise NotImplementedError('on_return is not implemented')
 
-    msg = {'result': replydata, 'failure': failure, 'ending': False, }
-    send_direct(connection, msg_id, msg)
-    msg = {'result': None, 'failure': None, 'ending': True, }
-    send_direct(connection, msg_id, msg)
+    _log.debug('replying to RPC message `%s`', msg_id)
+
+    with log_time(
+            _log.debug, 'replied to message `%s` in %0.3f sec.', msg_id):
+        if on_return is not None:
+            raise NotImplementedError('on_return is not implemented')
+
+        msg = {'result': replydata, 'failure': failure, 'ending': False, }
+        send_direct(connection, msg_id, msg)
+        msg = {'result': None, 'failure': None, 'ending': True, }
+        send_direct(connection, msg_id, msg)
 
 
 def _delegate_apply(delegate, context, method, args):
@@ -96,13 +101,13 @@ def _delegate_apply(delegate, context, method, args):
     return func(context=context, **args)
 
 
-def process_rpc_message(connection, delegate, body, reraise=False):
+def process_rpc_message(connection, delegate, body):
     msgid, ctx, method, args = context.parse_message(body)
 
-    _log.debug('processing message `%s`: using %s(...)', msgid, method)
+    _log.debug('processing RPC message `%s`: using %s(...)', msgid, method)
 
     with log_time(
-            _log.debug, 'processed message `%s` in %0.3f sec.', msgid):
+            _log.debug, 'processed RPC message `%s` in %0.3f sec.', msgid):
         try:
             ret = _delegate_apply(delegate, ctx, method, args)
         except Exception:
@@ -111,12 +116,6 @@ def process_rpc_message(connection, delegate, body, reraise=False):
                 tbfmt = traceback.format_exception(exc_typ, exc_val, exc_tb)
                 ret = (exc_typ.__name__, str(exc_val), tbfmt)
                 reply(connection, msgid, failure=ret)
-            if reraise:
-                raise exc_typ, exc_val, exc_tb
         else:
             if msgid:
-                _log.debug('replying to message `%s`', msgid)
-                with log_time(
-                        _log.debug,
-                        'replied to message `%s` in  %0.3f sec.', msgid):
-                    reply(connection, msgid, replydata=ret)
+                reply(connection, msgid, replydata=ret)
